;
;
;
;    Resea (x86_64/assembly.asm)
;    Mafice <mafice@mflabo.net>
;
;
;

[bits 64]



;
;  Function: asm_startupInterpreter
;
;    インタプリタを起動する
;
;  Parameters:
;
;    rdi  - コードのアドレス
;    rsi  - コードセグメントレジスタ
;    rdx  - データセグメントレジスタ
;    rcx  - スタックポインタ
;    r8   - RFLAGSレジスタ
;
;  Returns:
;
;    なし
;

global asm_startupInterpreter
asm_startupInterpreter:


add rdx, 3
add rsi, 3

push qword rdx  ;SS
push qword rcx  ;RSP
push qword r8   ;RFLAGS
push qword rsi  ;CS
push qword rdi  ;RIP

mov  ax, dx
mov  ds, ax
mov  es, ax
mov  fs, ax
mov  gs, ax

iretq




;
;  Function: asm_hlt
;
;    HLT命令を実行する
;
;  Parameters:
;
;    なし
;
;  Returns:
;
;    なし
;

global asm_hlt
asm_hlt:

hlt
ret




;
;  Function: asm_cli
;
;    CLI命令を実行する
;
;  Parameters:
;
;    なし
;
;  Returns:
;
;    なし
;

global asm_cli
asm_cli:

cli
ret




;
;  Function: asm_sti
;
;    STI命令を実行する
;
;  Parameters:
;
;    なし
;
;  Returns:
;
;    なし
;

global asm_sti
asm_sti:

sti
ret




;
;  Function: asm_out8
;
;    OUT命令を実行する
;
;  Parameters:
;
;    sil - データ
;    di  - ポート
;
;  Returns:
;
;    なし
;

global asm_out8
asm_out8:

mov  al, sil
mov  dx, di
out  dx, al
ret




;
;  Function: asm_in8
;
;    IN命令を実行する
;
;  Parameters:
;
;    di  - ポート
;
;  Returns:
;
;    読み取ったデータ
;

global asm_in8
asm_in8:

mov  rax,0
mov  dx, di
in   al, dx
ret




;
;  Function: asm_lgdt
;
;    LGDT命令を実行する
;
;  Parameters:
;
;    rdi  - LGTRの内容へのアドレス
;
;  Returns:
;
;    なし
;

global asm_lgdt
asm_lgdt:

lgdt [rdi]
ret




;
;  Function: asm_lidt
;
;    LIDT命令を実行する
;
;  Parameters:
;
;    rdi  - LDTRの内容へのアドレス
;
;  Returns:
;
;    なし
;

global asm_lidt
asm_lidt:

lidt [rdi]
ret




;
;  Function: asm_ltr
;
;    LTR命令を実行する
;
;  Parameters:
;
;    di  - TSSセレクタ
;
;  Returns:
;
;    なし
;

global asm_ltr
asm_ltr:

ltr di
ret




;
;  Function: asm_setCR3
;
;    CR3レジスタをセットする
;
;  Parameters:
;
;    rdi  - CR3レジスタのデータ
;
;  Returns:
;
;    なし
;

global asm_setCR3
asm_setCR3:

mov  rax, rdi
mov  cr3, rax
ret




;
;  Function: asm_loadCR2
;
;    CR2レジスタを読み込む
;
;  Parameters:
;
;    なし
;
;  Returns:
;
;    CR2レジスタのデータ
;

global asm_loadCR2
asm_loadCR2:

mov rax, cr2
ret

